// constants won't change. They're used here to set pin numbers:
const int buttonPin = 2;    // input:  a "1" means button is pushed
const int chargePin = 4;    // output: a "1" will start charging the inductor
const int resetPin  = 5;    // output: a "0" on this pin resets the 74HCT74
const int pulsedPin = 7;    // input:  a "1" on this pin means the 74HCT74 has captured a pulse
const int voltagePin = A0;  // analog input, high voltage
const int currentPin = A2;  // analog input, current

float Tinp;               // input time before times check
float Tmin = 1.0;         // minimum time
float Tmax = 10.0;        // maximum time
float Tstep;              // repeat step time
float Tx;                 // running repeat time
int   nsteps = 5;         // number of steps
int   ninp;               // input nsteps before check
int   i;                  // repeat loop counter
long  Tcharge = 10000;    // default charge time * 1000 (so 10ms)
int   pinread;            // holds status of push button
int   keyread;            // selection variable from menu
int   ADCvoltage;         // direct read from ADC voltage 0 -- 1023
int   ADCcurrent;         // direct read from ADC current 0 -- 1023
float voltage;            // real voltage converted from ADCvoltage
float current;            // real current converted from ADCcurrent
int   table_sel = 0;      // determines format of voltage and current printing, if table_sel = 1 it is directly importable in exel
          

// the setup function runs once when you press reset or power the board
void setup() {
  // initialize digital I/O's
  pinMode(buttonPin, INPUT);
  pinMode(chargePin, OUTPUT);
  pinMode(resetPin, OUTPUT);
  pinMode(pulsedPin, INPUT);
  // standbye state of outputs
  digitalWrite(chargePin, LOW);
  digitalWrite(pulsedPin, HIGH);
  // make reset pin high
  digitalWrite(resetPin, HIGH);
  //  
  Serial.begin(9600);
 }

// the loop function runs over and over again forever
void loop() {

  Serial.println();
  Serial.print("1. Single charge time (ms):     ");
  Serial.println(float(Tcharge)/1000.0);
  Serial.println("2. Single Fire! ");
  Serial.print("3. Min repeat charge time (ms): ");
  Serial.println(Tmin);
  Serial.print("4. Max repeat charge time (ms): ");
  Serial.println(Tmax);
  Serial.print("5. Number of steps >= 2!:       ");
  Serial.println(nsteps);
  Serial.println("6. Repeat fire! ");    
  Serial.println("Make your choice");
    
  //Wait until button is released
  pinread = digitalRead(buttonPin);
  while (pinread != 0) { 
    delay(10);
    pinread = digitalRead(buttonPin);
  }
  delay(10); 
  
  // Wait till button is pushed, or a number for the settigs is entered
  // on exit keyread == 0 if button is pressed or number 1....n is number is pressed
  pinread = digitalRead(buttonPin);
  while (pinread != 1 && Serial.available() == 0) { 
    delay(10);
    pinread = digitalRead(buttonPin);
  }
  if (pinread == 1) keyread = 0; 
  else {
     keyread = Serial.parseInt();
     clearbuffer();
  }

  // Parse the selection
  if ((keyread == 0) || (keyread == 2)) {
    table_sel = 0;  // voltage and current printed in one line
    Single_fire();
  }
  else if (keyread == 1) {
    Change_Tcharge();
  } 
  else if (keyread == 3) {
    Change_Tmin();
  } 
  else if (keyread == 4) {
    Change_Tmax();
  }
  else if (keyread == 5) {
    Change_nsteps();
  } 
  else if (keyread == 6) {
    table_sel = 1;  // voltage and current printed as a table
    Serial.println(" ");
    Serial.println("T(ms)   V(V)      I(A) ");
    Repeat_fire();
  }      
  else {
    Serial.println("Inalid input!");    
  }    
  delay(100);
}

  //Clear the input buffer from remaining CR/LF etc.
  void clearbuffer(){
      while (Serial.available()){
      Serial.read();
      }
  }

  //Change single fire time
  //This routine returns the time in ms*1000 as a long
  //So if 2.34 is entered, Tcharge = 2340
  void Change_Tcharge() {
    Serial.println(" ");
    Serial.println("Enter charge time in ms ");
    while (Serial.available() == 0){
    }
    Tinp = Serial.parseFloat();
    if ( (Tinp < 0) || (Tinp > 50)) Serial.println("Error, 0ms < charge time < 50ms");
      else
      Tcharge = long(Tinp*1000);
    clearbuffer();      
  }

  //Change min. repeat fire time
  //This routine returns the time in Tmin in ms as float
  void Change_Tmin() {
    Serial.println(" ");
    Serial.println("Enter minimum repeat charge time in ms ");
    while (Serial.available() == 0){
    }
    Tinp = Serial.parseFloat();
    if ( (Tinp < 0) || (Tinp > 50)) Serial.println("Error, 0ms < time < 50ms");
      else
      Tmin = Tinp;
    clearbuffer();      
  }

  //Change max. repeat fire time
  //This routine returns the time in Tmax in ms as float
  void Change_Tmax() {
    Serial.println(" ");
    Serial.println("Enter maximum repeat charge time in ms ");
    while (Serial.available() == 0){
    }
    Tinp = Serial.parseFloat();
    if ( (Tinp < 0) || (Tinp > 50)) Serial.println("Error, 0ms < time < 50ms");
      else
      Tmax = Tinp;
    clearbuffer();      
  }

  //Change number of repeat steps
  //This routine returns the number of repeat steps in nsteps as integer
  void Change_nsteps() {
    Serial.println(" ");
    Serial.println("Enter number of repeat steps >= 2! ");
    while (Serial.available() == 0){
    }
    ninp = Serial.parseInt();
    if ( (ninp < 2) || (ninp > 20)) Serial.println("Error, 2 < nsteps < 20");
      else
      nsteps = ninp;
    clearbuffer();      
  }

  //Fire
  void Single_fire(){ 
    // first reset the capture flipflop with an active low 
    digitalWrite(resetPin, LOW);
    delay(5);
    digitalWrite(resetPin, HIGH);
    delay(5);
    pinread = digitalRead(pulsedPin); 
//    pinread = 0; // <<<<<<<<<< weghalen
    if (pinread != 0) Serial.println("capture flipflop failed to reset");     
    else {
      //Fire!
      //Depending on value of Tcharge issue long or short pulse
      if (Tcharge <= 15000) {
        digitalWrite(chargePin, HIGH);
        delayMicroseconds(Tcharge);
        digitalWrite(chargePin, LOW); 
      }
      else {
        digitalWrite(chargePin, HIGH);
        delay(Tcharge/1000);
        digitalWrite(chargePin, LOW); 
      }
      //Within 5 ms, the capture flip flop should be set, if not error]
      delay(5);
      pinread = digitalRead(pulsedPin); 
 //     pinread = 1; // <<<<<<<<<< weghalen
      if (pinread != 1) Serial.println("failed to capture within 5 ms");
      else {
      ADCvoltage  = analogRead(voltagePin);
      ADCcurrent  = analogRead(currentPin);  
      // voltage = (ADCvoltage/1024)*5*1000 = 4.8828*ADC
      voltage = 4.8828*float(ADCvoltage);
      // current = ADCcurrent/((4.7/5)*1024)  = ADC/963
      current = (float(ADCcurrent)/963)*1000;
      if (table_sel == 0) {
        Serial.print("V = ");
        Serial.print(voltage);
        Serial.print("  I = ");
        Serial.println(current);      
      }
      else {
        //if (Tcharge < 10000) Serial.print(" "); // add space to nicely align text
        Serial.print(float(Tcharge)/1000.0);
        Serial.print(" ");
        Serial.print(voltage);
        Serial.print(" ");
        Serial.println(current);        
      }
      }
     }
    }  

  //Repeat fire
  void Repeat_fire() {
    if (Tmin >= Tmax) Serial.println("Error, Tmax must be larger than Tmin");
    else {
      Tx = Tmin;
      Tstep = (Tmax-Tmin)/int(nsteps-1);
      i = 0;
      while (i < nsteps) {
        Tcharge = long(Tx*1000);
        delay(500);
        Single_fire();
        Tx = Tx + Tstep;
        i = i+1;
        delay(2000);
      }
    }
  }
 
